package org.netkernel.mod.architecture.bits;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.mod.architecture.MenuAccessor;
import org.netkernel.mod.architecture.Utils;

public class StaticStructureDiagramViewer extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode defaults= aContext.source("arg:defaults",IHDSNode.class);
		String title=(String)defaults.getFirstValue("title");

		String url="/tools/ae/view/StaticStructureDiagram?";
		String rootHash=null;
		for (IHDSNode rootNode: defaults.getNodes("root"))
		{	if (rootHash==null)
			{	rootHash=(String)rootNode.getValue();
			}
			url+="&root="+rootNode.getValue();
		}
		
		String depth=(String)defaults.getFirstValue("depth");
		if (depth==null)
		{	depth="all";
		}
		String lib=(String)defaults.getFirstValue("lib");
		if (lib==null)
		{	lib="hide";
		}
		String module=(String)defaults.getFirstValue("module");
		if (module==null)
		{	module="hide";
		}
		
		String svgId="svg"+Utils.getUniqueId();
		
		StringBuilder script=new StringBuilder();
		script.append("<!--\n");
		script.append("$('#"+svgId+"').ready(function() {");
		script.append("initLinksRaw('.el-module','gMenuModule'); ");
		script.append("initLinksRaw('.el-space','gMenuSpace'); ");
		script.append("doSVGUpdate('"+svgId+"' ); ");
		script.append("}); ");
		script.append("\n-->");
		
		HDSBuilder b=new HDSBuilder();
		b.pushNode("div");
		if (title!=null)
		{	b.addNode("@tabtitle",title);
		}

		MenuAccessor.addExplorerTemplate(b,aContext,rootHash,"StaticStructureDiagram",null);
		
		b.addNode("@tabicon","ui-icon-ae ui-icon-ae-ssd");
		
		b.pushNode("script",script.toString());
		b.addNode("@type","text/javascript");
		b.popNode();
		
		b.pushNode("input");
		b.addNode("@type", "button");
		b.addNode("@onclick", "javascript:doSmaller('"+svgId+"')");
		b.addNode("@value", "-");
		b.popNode();
		b.pushNode("input");
		b.addNode("@type", "button");
		b.addNode("@onclick", "javascript:doLarger('"+svgId+"')");
		b.addNode("@value", "+");
		b.popNode();
		
		b.addNode("span"," modules:");
		b.pushNode("select");
		b.addNode("@id", svgId+"module");
		b.addNode("@onchange", "javascript:doSVGUpdate('"+svgId+"')");
		writeOption(b,"hide",module);
		writeOption(b,"show",module);
		b.popNode();

		b.addNode("span"," libraries:");
		b.pushNode("select");
		b.addNode("@id", svgId+"lib");
		b.addNode("@onchange", "javascript:doSVGUpdate('"+svgId+"')");
		writeOption(b,"hide",lib);
		writeOption(b,"merge",lib);
		writeOption(b,"show",lib);
		b.popNode();

		b.addNode("span"," depth:");
		b.pushNode("select");
		b.addNode("@id", svgId+"depth");
		b.addNode("@onchange", "javascript:doSVGUpdate('"+svgId+"')");
		writeOption(b,"all",depth);
		writeOption(b,"module",depth);
		writeOption(b,"1",depth);
		writeOption(b,"2",depth);
		writeOption(b,"3",depth);
		writeOption(b,"4",depth);
		writeOption(b,"5",depth);
		writeOption(b,"6",depth);
		writeOption(b,"7",depth);
		writeOption(b,"8",depth);
		writeOption(b,"9",depth);
		b.popNode();

		String shading="full";
		b.addNode("span"," shading:");
		b.pushNode("select");
		b.addNode("@id", svgId+"shading");
		b.addNode("@onchange", "javascript:doSVGUpdate('"+svgId+"')");
		writeOption(b,"full",shading);
		writeOption(b,"off",shading);
		b.popNode();
		
		b.addNode("span"," ");
		b.pushNode("input");
		b.addNode("@type","button");
		b.addNode("@value","download svg");
		b.addNode("@onclick","javascript:downloadSVG('"+svgId+"')");
		b.popNode();
		
		b.pushNode("div");
		b.addNode("@id",svgId);
		b.addNode("@class","svgdiv");
		b.addNode("@style", "width: 100%; overflow: auto");
		b.addNode("@url", url);
		b.addNode("@scale", "1.0");
		b.popNode();
		
		IHDSNode root=b.getRoot();
		INKFResponse resp=aContext.createResponseFrom(root);
		resp.setHeader("WrappedControlPanel", "Static Structure Diagram");
		resp.setHeader("WrappedControlPanelSubtitle", "");
		resp.setHeader("WrappedControlPanelIcon", "/tools/ae/pub/images/48/Space48.png");
		
	}
	
	private static void writeOption(HDSBuilder aBuilder, String aOption, String aCurrent)
	{	aBuilder.pushNode("option",aOption);
		if (aOption.equals(aCurrent))
		{	aBuilder.addNode("@selected", "true");
		}
		aBuilder.popNode();
	}
}
